//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1FilePreviewViewController.h"

#import <QuickLook/QuickLook.h>

#import "U1FileNode.h"
#import "U1FilesClient.h"
#import "U1LocalFileInfo.h"


@interface U1FilePreviewViewController () <QLPreviewControllerDataSource, QLPreviewControllerDelegate>
@property (retain) U1FileNode *node;
@property (retain) U1FilesClient *filesClient;
@property (nonatomic, retain) U1LocalFileInfo *fileInfo;
@property (retain) QLPreviewController *previewController;
@end


@implementation U1FilePreviewViewController

@synthesize containerView, downloadContainerView, downloadProgressView, previewContainerView;
@synthesize node, filesClient, fileInfo, previewController;

- (id)initWithNode:(U1FileNode *)aNode;
{
	self = [super initWithNibName:nil bundle:nil];
	if (!self)
		return nil;
	node = [aNode retain];
	filesClient = [[U1FilesClient sharedFilesClient] retain];
	return self;
}

- (void)dealloc;
{
	[containerView release];
	[downloadContainerView release];
	[downloadProgressView release];
	[previewContainerView release];
	[node release];
	[filesClient release];
	self.fileInfo = nil;
	[previewController release];
	[super dealloc];
}

- (void)viewDidLoad;
{
	[super viewDidLoad];

	self.fileInfo = [self.filesClient downloadFileForNode:self.node completionBlock:^(NSError *error) {
		
	}];
}

- (void)viewDidUnload;
{
	self.fileInfo = nil;
	
	[containerView release], containerView = nil;
	[downloadContainerView release], downloadContainerView = nil;
	[downloadProgressView release], downloadProgressView = nil;
	[previewContainerView release], previewContainerView = nil;
	[super viewDidUnload];
}

- (void)setFileInfo:(U1LocalFileInfo *)value;
{
	if (fileInfo == value)
		return;
	
	[value retain];
	
	[fileInfo removeObserver:self forKeyPath:@"downloadPercentage"];
	[fileInfo removeObserver:self forKeyPath:@"downloading"];
	[fileInfo removeObserver:self forKeyPath:@"downloaded"];
	
	[fileInfo autorelease];
	fileInfo = value;
	
	[fileInfo addObserver:self forKeyPath:@"downloadPercentage" options:0 context:self.fileInfo];
	[fileInfo addObserver:self forKeyPath:@"downloading" options:0 context:self.fileInfo];
	[fileInfo addObserver:self forKeyPath:@"downloaded" options:NSKeyValueObservingOptionInitial context:self.fileInfo];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context;
{
	if (context == self.fileInfo && [keyPath isEqualToString:@"downloadPercentage"])
	{
		[self.downloadProgressView setProgress:[self.fileInfo downloadPercentage]];
	}
	else if (context == self.fileInfo && [keyPath isEqualToString:@"downloading"])
	{
		if ([self.fileInfo isDownloading])
		{
		}
	}
	else if (context == self.fileInfo && [keyPath isEqualToString:@"downloaded"])
	{
		if ([self.fileInfo isDownloaded])
		{
			[UIView transitionFromView:self.downloadContainerView toView:self.previewContainerView duration:0.4 options:UIViewAnimationOptionShowHideTransitionViews completion:NULL];
			
			self.previewController = [[[QLPreviewController alloc] init] autorelease];
			self.previewController.dataSource = self;
			self.previewController.delegate = self;
			
			UIView *previewView = self.previewController.view;
			previewView.frame = [self.previewContainerView bounds];
			[self.previewController viewWillAppear:NO];
			[self.previewContainerView addSubview:previewView];
			[self.previewController viewDidAppear:NO];
			
		}
		else
		{
			[UIView transitionFromView:self.previewContainerView toView:self.downloadContainerView duration:0.4 options:UIViewAnimationOptionShowHideTransitionViews completion:NULL];
		}
	}
	else
	{
		[super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
	}
}


#pragma mark Actions

- (IBAction)deleteItem:(UIBarButtonItem*)sender;
{
	[self.filesClient deleteNode:self.node completionBlock:^(NSError *error) {
		
	}];
	NSArray *viewControllers = [self.navigationController viewControllers];
	if ([viewControllers count] > 2)
	{
		UIViewController *destinationController = [viewControllers objectAtIndex:[viewControllers count] - 3];
		[self.navigationController popToViewController:destinationController animated:YES];
	}
	else
	{
		[self.navigationController popToRootViewControllerAnimated:YES];
	}
}

- (IBAction)openIn:(UIBarButtonItem*)sender;
{
	UIDocumentInteractionController *docController = [UIDocumentInteractionController interactionControllerWithURL:fileInfo.fileURL];
	[docController presentOpenInMenuFromBarButtonItem:sender animated:YES];
}


#pragma mark QLPreviewControllerDelegate Methods

- (NSInteger) numberOfPreviewItemsInPreviewController: (QLPreviewController *) controller;
{
	return 1;
}

- (id <QLPreviewItem>) previewController: (QLPreviewController *) controller previewItemAtIndex: (NSInteger) index;
{
	return [self.fileInfo fileURL];
}

@end
