//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1UploadOperation.h"

#import "U1Asset.h"
#import "U1DataRepository.h"
#import "U1FilesService.h"
#import "U1FileNode.h"
#import "U1FolderNode.h"
#import "U1LocalFileInfo.h"


@interface U1UploadOperation ()
@property (getter=isExecuting) BOOL executing;
@property (getter=isFinished) BOOL finished;
- (void)finishExecuting;
- (void)beginUpload;
@end


@implementation U1UploadOperation

@synthesize executing, finished, dataProvider, fileNode, mimetype, error, progressBlock, fileInfo;

- (id)init;
{
	self = [super init];
	if (!self)
		return nil;
	fileInfo = [[U1LocalFileInfo alloc] init];
	return self;
}

- (BOOL)isConcurrent;
{
	return YES;
}

- (void)finishExecuting;
{
	[self willChangeValueForKey:@"isExecuting"];
	[self willChangeValueForKey:@"isFinished"];
	self.finished = YES;
	self.executing = NO;
	[self didChangeValueForKey:@"isFinished"];
	[self didChangeValueForKey:@"isExecuting"];
//	[UONetworkStatusCoordinator removeNetworkActivity];
}

- (void)start;
{
	[self willChangeValueForKey:@"isExecuting"];
	self.executing = YES;
	[self didChangeValueForKey:@"isExecuting"];

	[self beginUpload];
}

- (void)beginUpload;
{
	if ([self isCancelled])
	{
		[self finishExecuting];
		return;
	}
	
	dispatch_async(dispatch_get_main_queue(), ^(void) {

		[self.fileInfo setUploadPercentage:0.0];
		[self.fileInfo setUploading:YES];
		[self.fileInfo setUploaded:NO];

		U1FilesService *filesService = [U1FilesService sharedFilesService];
		fileInfo.uploading = YES;
		[filesService uploadContentData:self.dataProvider forNode:self.fileNode withContentType:self.mimetype progressBlock:^(long long uploaded, long long totalLength) {
			
			dispatch_async(dispatch_get_main_queue(), ^{
				[self.fileInfo setUploadPercentage:((double)uploaded / (double)totalLength)];
			});
			
			if (self.progressBlock)
				self.progressBlock(uploaded, totalLength);
			
		} completionBlock:^(NSDictionary *updatedNodeInfo, NSError *filesServiceError) {
			
			dispatch_async(dispatch_get_main_queue(), ^{
				[self.fileInfo setUploadPercentage:1.0];
				[self.fileInfo setUploading:NO];
				[self.fileInfo setUploaded:YES];
			});
			
			if (filesServiceError != nil)
			{
				self.error = filesServiceError;
				[self beginUpload];
			}
			else
			{
				[self.fileNode updatePropertiesFromJSONDictionary:updatedNodeInfo];
				[self finishExecuting];
			}
		}];
	});
}

- (void)dealloc
{
	[dataProvider release];
	[fileNode release];
	[mimetype release];
	[error release];
	[progressBlock release];
	[fileInfo release];
    [super dealloc];
}

@end
